<?php

namespace Drupal\clean_files_entity;

use Drupal\Core\Database\Query\Condition;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;

/**
 * Class of CleanFilesEntity.
 */
class CleanFilesEntity {

  use DependencySerializationTrait;

  /**
   * Maximum number per run.
   *
   * @var int|mixed
   */
  protected $max;

  /**
   * Database object.
   *
   * @var \Drupal\Core\Database\Connection|object|null
   */
  protected $database;

  /**
   * Constructs a new instance of the class.
   */
  public function __construct() {
    $this->database = \Drupal::getContainer()->get('database');
    $this->max = $GLOBALS['config']['clean_files_entity']['max'] ?? 50;
  }

  /**
   * Deletes unused files associated with a specific entity type.
   *
   * @param string $entityType
   *   The type of entity to check for unused files.
   */
  public function run($entityType) {
    $query = $this->database->select('file_usage', 'f');
    $query->fields('f', ['fid']);
    $query->condition('f.type', $entityType);

    $query->leftJoin('node_field_data', 'n', 'n.nid = f.id');
    $query->isNull('n.nid');

    $fids = $query->range(0, $this->max)->execute()->fetchCol();
    if (!empty($fids)) {
      $this->delete($fids);
    }
  }

  /**
   * Deletes unused files from specified folders.
   *
   * @param array $folders
   *   An array of folder paths to check for unused files.
   */
  public function runFolder(array $folders = []) {
    $query = $this->database->select('file_managed', 'f');
    $query->fields('f', ['fid']);
    $db_or = new Condition('OR');
    foreach ($folders as $folder) {
      $db_or->condition('f.uri', $query->escapeLike($folder) . '%', 'LIKE');
    }
    $query->condition($db_or);

    $query->leftJoin('file_usage', 'u', 'u.fid = f.fid');
    $query->isNull('u.fid');

    $fids = $query->range(0, $this->max)->execute()->fetchCol();
    if (!empty($fids)) {
      $this->delete($fids);
    }
  }

  /**
   * Deletes files and their usage entries from the database.
   */
  public function delete($fids) {
    $this->database->delete('file_usage')
      ->condition('fid', $fids, 'in')
      ->execute();
    $names = [];
    foreach (\Drupal::entityTypeManager()->getStorage('file')->loadMultiple($fids) as $file) {
      $names[] = $file->getFileUri();
      $file->delete();
    }
    \Drupal::logger('clean_files_entity')->notice(count($names) . ': <pre>' . print_r($names, TRUE) . '</pre>');
  }

}
